'use strict';

module.exports = {
  api,
  listFiles,
  getFiles,
  updateFiles,
  getFileId,
  getFileName,
  getMime,
  listChildren,
  _resolveParamsId,
  _resolveFId,
};

/**
 * Base function for making requests to the Google Drive API.
 * @see Consult
 * [Google Drive API reference]{@link https://developers.google.com/drive/api/v3/reference}
 * for information on the resources and methods available
 * as well as the API methods' parameters.
 * @param {string} resource - Name of API resource
 * @param {string} method - One of the API resource's methods
 * @param {Object} params - Method parameters
 * @return {Object} Data from response
 * @example
 * // Use the files resource and list method to
 * // get ids of files with the name "fileName"
 * utilsGDrive.api("files", "list", {
 *   q: "name = 'fileName'",
 *   fields: "files(id)"
 * });
 */
function api(resource, method, params) {
  return new Promise((resolve, reject) => {
    this.drive[resource][method](params, (err, response) => {
      if (err) {
        reject(err);
        return;
      }
      resolve(response.data);
    });
  });
}

/**
 * Makes a request to the API's files resource using
 * its list method, which is primarily used to search for files/folders
 * using the <code>q</code> method parameter.
 * @see Wraps {@link api}
 * @param {Object} params - Method parameters,
 * two of which are highlighted below.
 * @param {string} [params.q] - Query string used to search for files/folders
 * @param {string} [params.fields] - Data fields to request.
 * Should follow the pattern <code>"files(fiel1, field2,...)"</code>.
 * Default value is <code>"files(name, id, mimeType)"</code>
 * @param {boolean} [ignoreTrash=true] - Whether to include trash
 * as a search location
 * @return {Object} Data from response
 * @example
 * // search for files with the name "fileName" and get their ids
 * utilsGDrive.listFiles({
 *   q: "name = 'fileName'",
 *   fields: "files(id)"
 * });
 */
function listFiles(params, ignoreTrash=true) {
  if (!params.fields) params.fields = 'files(name, id, mimeType)';
  if (params.q) {
    const regEx = new RegExp('(and)? trashed ?= ?(true|false)');
    const matches = regEx.exec(params.q);
    if (matches) params.q = params.q.replace(matches[0], '');
    params.q += ' and trashed=' + !ignoreTrash;
  }
  return this.api('files', 'list', params).catch((err) => console.error(err));
}

/**
 * Makes a request to the files resource using its get method,
 * which can be used to get information on files and/or download them.
 * @see Wraps {@link api}
 * @param {Object} params - Method parameters.
 * <code>fileId</code> is a required property.
 * @param {string} params.fileId - File/folder id
 * @return {Object} Data from response
 * @throws Throws an error when file/folder id isn't specified.
 * More specifically, when <code>params.fileId</code> is falsy
 * @example
 * // get MIME type of the file whose id is "fileId"
 * utilsGDrive.getFiles({
 *   fileId: "fileId",
 *   fields: "mimeType"
 * });
 */
function getFiles(params) {
  if (!params.fileId) {
    const err = new this.Error('File id not specified.');
    // wrapping in Promise.resolve ensures chainability
    return Promise.resolve(console.error(err));
  }
  if (!params.fields) params.fields = 'name, id, mimeType';
  return this.api('files', 'get', params).catch((err) => console.error(err));
}

/**
 * Makes a request to the API's files resource using
 * its update method, which can be used
 * to change a file or folders's name or to move a file to a different location
 * within Google Drive.
 * @see Wraps {@link api}
 * @param {Object} params - Method parameters
 * <code>fileId</code> is a required property.
 * @param {string} params.fileId - File/folder id
 * @return {Object} Data from response
 * @throws Throws an error when file/folder id isn't specified.
 * More specifically, when <code>params.fileId</code> is falsy
 * @example
 * // change the name of a file whose id is "fileId" to "newFileName"
 * utilsGDrive.updateFiles({
 *   fileId: "fileId"
 *   resource: {name: "newFileName"}
 * });
 */
function updateFiles(params) {
  if (!params.fileId) {
    const err = new this.Error('File id not specified.');
    // wrapping in Promise.resolve ensures chainability
    return Promise.resolve(console.error(err));
  }
  return this.api('files', 'update', params).catch((err) => console.error(err));
}

/**
 * Resolves id of file/folder using <code>params</code>.
 * @param {Object|string} params - An object having a property whose name
 * ends in either "Name" or "Id". Can also be a string containing the
 * the file/folder id
 * @return {string} File/folder id
 * @access private
 */
async function _resolveParamsId(params) {
  if (typeof params === 'string') return params;
  let propPrefix;
  for (const key in params) {
    if (key.indexOf('Name') + key.indexOf('Id') > -2) {
      propPrefix = key.replace('Id', '').replace('Name', '');
      break;
    }
  }
  // id given
  if (params[propPrefix + 'Id']) {
    return params[propPrefix + 'Id'];
    // name given
  } else if (params[propPrefix + 'Name']) {
    return this.getFileId(params[propPrefix + 'Name']);
    // default to root if neither id nor name given
  } else {
    return 'root';
  }
}

/**
 * Gets file/folder id given name.
 * @param {Object|string} identifiers - Function parameters.
 * A string containing the file/folder name may be passed instead.
 * @param {string} identifiers.fileName - File/folder name
 * @param {string} [identifiers.parentId] - Parent id
 * @param {string} [identifiers.parentName] - Parent name
 * @return {string} File/folder id
 * @throws Throws an error if exactly one file/folder isn't found
 * given identifiers specified
 * @example
 * // get id of file whose name is "fileName"
 * // and whose parent is named "folderName"
 * utilsGDrive.getFileId({
 *   fileName: "fileName",
 *   parentName: "folderName"
 * })
 *
 * // get id of file whose name is "fileName"
 * // assuming no other files/folders in Google Drive
 * // are also named "fileName"
 * utilsGDrive.getFileId("fileName");
 */
async function getFileId({fileName, parentId, parentName}) {
  if (typeof(arguments[0]) === 'string') {
    fileName = arguments[0];
  }

  let q = 'name="' + fileName + '"';
  if (parentId || parentName) {
    parentId = await this._resolveParamsId({parentId, parentName});
    q += ' and "' + parentId + '" in parents';
  }

  const responseData = await this.listFiles({q, fields: 'files(id)'});
  if (responseData) {
    let e;
    const nFiles = responseData.files.length;
    if (nFiles > 1) {
      e = new this.Error('Multiple files found: '+
        fileName+
        '. Consider specifying parent.');
    } else if (nFiles === 0) {
      e = new this.Error('No files found matching identifiers specified.');
    }
    if (e) return console.error(e);
    return responseData.files[0].id;
  }
}

/**
 * Gets file/folder name given id.
 * @param {string} fileId - File/folder id
 * @return {string} File/folder name
 * @example
 * // get name of file whose id is "fileId"
 * utilsGDrive.getFileName("fileId");
 */
async function getFileName(fileId) {
  const responseData = await this.getFiles({fileId, fields: 'name'});
  if (responseData) return responseData.name;
}

/**
 * Resolves file/folder id given object with data
 * identifying the file/folder.
 * @param {Object|string} identifiers - An object
 * used to identify a file or folder. Must have either
 * <code>fileId</code> or <code>fileName</code>.
 * This parameter also accepts a string containing the file/folder id.
 * @param {string} identifiers.fileId - File/folder id
 * @param {string} identifiers.fileName - File/folder name
 * @param {string} [identifiers.parentId] - Parent id
 * @param {string} [identifiers.parentName] - Parent name
 * @throws Throws an error if invalid property name given.
 * Valid property names are the ones above.
 * @return {string} File/folder id
 * @access private
 */
async function _resolveFId(identifiers) {
  if (typeof identifiers === 'string') return identifiers;
  if (identifiers.fileId) return identifiers.fileId;
  if (identifiers.folderId) return identifiers.folderId;

  const fIdentifiers = ['fileId', 'fileName'];
  const pIdentifiers = ['parentId', 'parentName'];
  const fParams = {};
  const pParams = {};
  for (const key in identifiers) {
    if (fIdentifiers.includes(key)) {
      fParams[key] = identifiers[key];
    } else if (pIdentifiers.includes(key)) {
      pParams[key] = identifiers[key];
    } else {
      throw new this.Error('Invalid property name: ' + key);
    }
  }

  let fName;
  if (fParams.fileName) fName = fParams.fileName;
  let parentId;
  if (Object.keys(pParams).length) {
    parentId = await this._resolveParamsId(pParams);
  }
  return this.getFileId({fileName: fName, parentId});
}

/**
 * Gets a file or folder's MIME type.
 * @param {Object|string} identifiers - An object
 * used to identify a file or folder. Must have either
 * <code>fileId</code> or <code>fileName</code>.
 * This parameter also accepts a string containing the file/folder id.
 * @param {string} identifiers.fileId - File/folder id
 * @param {string} identifiers.fileName - File/folder name
 * @param {string} [identifiers.parentId] - Parent id
 * @param {string} [identifiers.parentName] - Parent name
 * @return {string} MIME type
 * @example
 * // get MIME type of file whose id is "fileId"
 * utilsGDrive.getMime("fileId")
 *
 * // get MIME type of file whose id is "fileId"
 * // and that resides in folder whose id is "folderId"
 * utilsGDrive.getMime({
 *   fileId: "fileId",
 *   parentId: "folderId"
 * });
 */
async function getMime(identifiers) {
  let fileId;
  try {
    fileId = await this._resolveFId(identifiers);
  } catch (err) {
    return console.error(err);
  }
  const responseData = await this.getFiles({
    fileId,
    fields: 'mimeType',
  });
  if (responseData) return responseData.mimeType;
}

/**
 * Gets data on files in a folder.
 * @param {Object|string} identifiers - An object
 * used to identify a file or folder. Must have either
 * <code>fileId</code> or <code>fileName</code>.
 * This parameter also accepts a string containing the file/folder id.
 * @param {string} identifiers.fileId - File/folder id
 * @param {string} identifiers.fileName - File/folder name
 * @param {string} [identifiers.parentId] - Parent id
 * @param {string} [identifiers.parentName] - Parent name
 * @param {string} [fields] - Data fields to request.
 * Should follow the pattern <code>"files(fiel1, field2,...)"</code>.
 * Default value is <code>"files(name, id, mimeType)"</code>
 * @return {Array} Each entry in the array returned
 * is an object containing data on a file
 * that resides in the folder specified
 * @example
 * // get names of files in the folder whose id is "folderId"
 * utilsGDrive.listChildren("folderId", "files(name)");
 *
 * // get ids of files in the folder whose name is "folderName"
 * // and whose parent has the id "parentId"
 * utilsGDrive.listChildren({
 *   fileName: "folderName",
 *   parentId: "parentId"
 * }, "files(id)");
 *
 */
async function listChildren(identifiers, fields = 'files(name, id, mimeType)') {
  let folderId;
  try {
    folderId = await this._resolveFId(identifiers);
  } catch (err) {
    return console.error(err);
  }
  const listFilesParams = {
    q: '"' + folderId + '" in parents',
    fields: fields,
  };
  const responseData = await this.listFiles(listFilesParams);
  if (responseData) return responseData.files;
}
